import org.codehaus.groovy.grails.commons.ConfigurationHolder as CH

import org.springframework.orm.hibernate3.support.OpenSessionInViewInterceptor
import org.springframework.web.servlet.handler.BeanNameUrlHandlerMapping
import org.springframework.web.servlet.handler.SimpleMappingExceptionResolver 
import org.springframework.web.servlet.view.JstlView
import org.springframework.web.servlet.i18n.LocaleChangeInterceptor

import com.burtbeckwith.grails.plugin.springmvc.JspViewResolver

class SpringmvcGrailsPlugin {

	
	def version = 0.1
	def dependsOn = [:]
	def author = 'Burt Beckwith'
	def authorEmail = 'beckwithb@studentsonly.com'
	def title = 'Grails Spring MVC Controller Plugin.'
	def description = 'Enables the use of Spring MVC controllers.'
	def documentation = 'http://burtbeckwith.com/blog/?p=71'

	def doWithSpring = {

		def config = CH.config.springmvc
		mvcOpenSessionInViewInterceptor(OpenSessionInViewInterceptor) {
			sessionFactory = ref('sessionFactory')
		}

		mvcLocaleChangeInterceptor(LocaleChangeInterceptor)

		def handlerInterceptors = [mvcOpenSessionInViewInterceptor, mvcLocaleChangeInterceptor]
		CH.config.springmvc.interceptors.each { interceptor ->
			handlerInterceptors << ref(interceptor)
		}
		mvcHandlerMapping(BeanNameUrlHandlerMapping) {
			detectHandlersInAncestorContexts = true
			order = 1
			interceptors = handlerInterceptors
		}

		mvcViewResolver(JspViewResolver) {
			viewClass = JstlView
			order = 1
			prefix = '/WEB-INF/jsp/'
			suffix = '.jsp'
		}

		handlerExceptionResolver(SimpleMappingExceptionResolver) {
			order = 1
			defaultErrorView = config.defaultErrorView ?: 'error' // default to WEB-INF/error.jsp
			if (config.exceptionMappings) {
				exceptionMappings = new Properties()
				config.exceptionMappings.each { key, value ->
					exceptionMappings.setProperty key, value
				}
			}
		}
	}

	def doWithWebDescriptor = { webXml ->

		def config = CH.config.springmvc
		String suffix = config.urlSuffix ?: 'action'

		def servlets = webXml.servlet
		def lastServlet = servlets[servlets.size() - 1]
		lastServlet + {
			servlet {
				'servlet-name'('SpringMVC')
				'servlet-class'('org.springframework.web.servlet.DispatcherServlet')
				'load-on-startup'(1)
				'init-param' {
					'param-name'('detectAllHandlerExceptionResolvers')
					'param-value'('false')
				}
			}
		}

		def mappings = webXml.'servlet-mapping'
		def lastMapping = mappings[mappings.size() - 1]
		lastMapping + {
			'servlet-mapping' {
				'servlet-name'('SpringMVC')
				'url-pattern'("*.$suffix")
			}
		}
	}

	def doWithApplicationContext = { applicationContext ->
		// not used		
	}

	def doWithDynamicMethods = { ctx ->
		// not used		
	}

	def onChange = { event ->
		// not used		
	}

	def onConfigChange = { event ->
		// not used		
	}
}
