package com.burtbeckwith.grails.plugin.springmvc;

import org.springframework.core.io.Resource;
import org.springframework.web.servlet.view.AbstractUrlBasedView;
import org.springframework.web.servlet.view.UrlBasedViewResolver;

/**
 * Workaround for Grails view resolver not being Ordered, so it ends up 2nd in the
 * resolver list and doesn't get a chance to resolve GSPs.
 *
 * @author Burt
 */
public class JspViewResolver extends UrlBasedViewResolver {

	/**
	 * {@inheritDoc}
	 * @see org.springframework.web.servlet.view.UrlBasedViewResolver#buildView(java.lang.String)
	 */
	@Override
	protected AbstractUrlBasedView buildView(final String viewName) throws Exception {
		AbstractUrlBasedView view = super.buildView(viewName);
		String url = view.getUrl();
		if (url.startsWith("/")) {
			url = url.substring(1);
		}
		Resource resource = getApplicationContext().getResource(url);
		if (resource != null && resource.exists()) {
			return view;
		}

		// either a real 404 or a GSP, so let the next resolver attempt it
		return null;
	}
}
